import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.Random;

public class TinyJavaPgbench {
	public static void main(String[] args) throws Exception {
		if (args.length != 2) {
			psrintUsage();
			System.exit(1);
		}
		int ntransactions = 0;
		try {
			ntransactions = Integer.parseInt(args[0]);
		} catch (NumberFormatException e) {
			psrintUsage();
			System.exit(1);
		}
		int commmitCount = 1;
		try {
			commmitCount = Integer.parseInt(args[1]);
		} catch (NumberFormatException e) {
			psrintUsage();
			System.exit(1);
		}

		Class.forName("org.postgresql.Driver");

		Connection conn = null;
		try {
			conn = DriverManager.getConnection("jdbc:postgresql:test", "test", "test");
			conn.setAutoCommit(false);

			Statement stmt = conn.createStatement();
			ResultSet rs = stmt.executeQuery("select count(*) from branches");
			rs.next();
			int scale = rs.getInt(1);
			rs.close();
			stmt.close();

			Random random = new Random(System.currentTimeMillis());
			int [] delta = new int[ntransactions];
			int [] aid = new int[ntransactions];
			int [] bid = new int[ntransactions];
			int [] tid = new int[ntransactions];
			for (int i = 0; i < ntransactions; i++) {
				delta[i] = random.nextInt(1000) + 1;
				aid[i] = random.nextInt(100000) + 1;
				bid[i] = random.nextInt(scale) + 1;
				tid[i] = random.nextInt(10) + 1;
			}
			
			// (README.pgbench_jisp)
			// :delta11000܂ł̒l闐C:aid  1100000܂
			// ̒l闐łDȉC̒l͂ꂼꂱ̃gUNV
			// ł͓lg܂D

			PreparedStatement pstmt1
				= conn.prepareStatement("update accounts set abalance = abalance + ? where aid = ?");

			// (README.pgbench_jisp)
			// ł1܂D
			PreparedStatement pstmt2
				= conn.prepareStatement("select abalance from accounts where aid = ?");

			// (README.pgbench_jisp)
			// :tid  110̊Ԃ̒lƂ闐łD
			PreparedStatement pstmt3
				= conn.prepareStatement("update tellers set tbalance = tbalance + ? where tid = ?");

			// (README.pgbench_jisp)
			// :bid  1 [XPOt@N^[]̊Ԃ̒l闐łD
			PreparedStatement pstmt4
				= conn.prepareStatement("update branches set bbalance = bbalance + ? where bid = ?");

			PreparedStatement pstmt5
				= conn.prepareStatement("insert into history(tid,bid,aid,delta) values(?,?,?,?)");

			long start = System.currentTimeMillis();
			for (int i = 0; i < ntransactions; i++) {
				pstmt1.setInt(1, delta[i]);
				pstmt1.setInt(2, aid[i]);
				pstmt1.execute();

				pstmt2.setInt(1, aid[i]);
				pstmt2.execute();

				pstmt3.setInt(1, delta[i]);
				pstmt3.setInt(2, tid[i]);
				pstmt3.execute();

				pstmt4.setInt(1, delta[i]);
				pstmt4.setInt(2, bid[i]);
				pstmt4.execute();

				pstmt5.setInt(1, tid[i]);
				pstmt5.setInt(2, bid[i]);
				pstmt5.setInt(3, aid[i]);
				pstmt5.setInt(4, delta[i]);
				pstmt5.execute();
				
				if (i % commmitCount == 0 || i == ntransactions - 1) {
					conn.commit();
				}
			}
			long end = System.currentTimeMillis();
			float tps = (float) ntransactions * 1000 / (end - start);
			System.out.println("commmitCount = " + commmitCount + ",tps = " + tps);
		} finally {
			if (conn != null) {
				conn.close();
			}
		}
	}
	
	private static void psrintUsage() {
		System.err.println("java TinyJavaPgbench ntransactions commmitCount");
	}
}